@@ -15,7 +15,7 @@
     </message>
     <message>
         <source>Copy the currently selected address to the system clipboard</source>
-        <translation>Copier l'adresse surlignée dans votre presse-papiers</translation>
+        <translation>Copier l'adresse sélectionnée dans le presse-papiers</translation>
     </message>
     <message>
         <source>&amp;Copy</source>
@@ -41,9 +41,25 @@
         <source>&amp;Delete</source>
         <translation>&amp;Supprimer</translation>
     </message>
+    <message>
+        <source>Choose the address to send coins to</source>
+        <translation>Choisissez une adresse où envoyer les bitcoins</translation>
+    </message>
+    <message>
+        <source>Choose the address to receive coins with</source>
+        <translation>Choisissez une adresse où recevoir les bitcoins</translation>
+    </message>
     <message>
         <source>C&amp;hoose</source>
-        <translation>C&amp;oisir</translation>
+        <translation>C&amp;hoisir</translation>
+    </message>
+    <message>
+        <source>Sending addresses</source>
+        <translation>Adresses d'envoi</translation>
+    </message>
+    <message>
+        <source>Receiving addresses</source>
+        <translation>Adresses de réception</translation>
     </message>
     <message>
         <source>Copy &amp;Label</source>
@@ -111,9 +127,13 @@
         <source>Decrypt wallet</source>
         <translation>Décrypter le porte-monnaie</translation>
     </message>
+    <message>
+        <source>Warning: If you encrypt your wallet and lose your passphrase, you will &lt;b&gt;LOSE ALL OF YOUR BITCOINS&lt;/b&gt;!</source>
+        <translation>Attention : Si vous chiffrez votre portefeuille et que vous perdez votre mot de passe vous &lt;b&gt; PERDREZ TOUS VOS BITCOINS&lt;/b&gt; !</translation>
+    </message>
     <message>
         <source>Wallet encrypted</source>
-        <translation>Porte-monnaie chiffré</translation>
+        <translation>Portefeuille chiffré</translation>
     </message>
     </context>
 <context>
@@ -131,7 +151,7 @@
     <name>BitcoinGUI</name>
     <message>
         <source>Sign &amp;message...</source>
-        <translation>Signer &amp;message...</translation>
+        <translation>Signer un &amp;message...</translation>
     </message>
     <message>
         <source>Synchronizing with network...</source>
@@ -416,7 +436,7 @@
     </message>
     <message>
         <source>Fee:</source>
-        <translation>Frais:</translation>
+        <translation>Frais :</translation>
     </message>
     <message>
         <source>Dust:</source>
@@ -478,6 +498,14 @@
         <source>Copy amount</source>
         <translation>Copier le montant</translation>
     </message>
+    <message>
+        <source>Copy transaction ID</source>
+        <translation>Copier l'ID de transaction</translation>
+    </message>
+    <message>
+        <source>Copy quantity</source>
+        <translation>Copier la quantité</translation>
+    </message>
     <message>
         <source>Copy fee</source>
         <translation>Copier les frais</translation>
@@ -1178,10 +1206,18 @@
         <source>&amp;Save Image...</source>
         <translation>&amp;Sauvegarder image</translation>
     </message>
+    <message>
+        <source>URI</source>
+        <translation>URI</translation>
+    </message>
     <message>
         <source>Address</source>
         <translation>Adresse</translation>
     </message>
+    <message>
+        <source>Amount</source>
+        <translation>Montant</translation>
+    </message>
     <message>
         <source>Label</source>
         <translation>Étiquette</translation>
@@ -1312,6 +1348,10 @@
         <source>S&amp;end</source>
         <translation>E&amp;voyer</translation>
     </message>
+    <message>
+        <source>Copy quantity</source>
+        <translation>Copier la quantité</translation>
+    </message>
     <message>
         <source>Copy amount</source>
         <translation>Copier le montant</translation>
@@ -1328,6 +1368,10 @@
         <source>Copy bytes</source>
         <translation>Copier les octets</translation>
     </message>
+    <message>
+        <source>or</source>
+        <translation>ou</translation>
+    </message>
     <message>
         <source>(no label)</source>
         <translation>(aucune étiquette)</translation>
@@ -1386,7 +1430,11 @@
     </context>
 <context>
     <name>SendConfirmationDialog</name>
-    </context>
+    <message>
+        <source>Yes</source>
+        <translation>Oui</translation>
+    </message>
+</context>
 <context>
     <name>ShutdownWindow</name>
     </context>
@@ -1525,6 +1573,10 @@
         <source>Comment</source>
         <translation>Commentaire </translation>
     </message>
+    <message>
+        <source>Amount</source>
+        <translation>Montant</translation>
+    </message>
     <message>
         <source>true</source>
         <translation>vrai</translation>
@@ -1547,6 +1599,10 @@
         <source>Date</source>
         <translation>Date</translation>
     </message>
+    <message>
+        <source>Type</source>
+        <translation>Type</translation>
+    </message>
     <message>
         <source>Label</source>
         <translation>Étiquette</translation>
@@ -1602,6 +1658,10 @@
         <source>Copy amount</source>
         <translation>Copier le montant</translation>
     </message>
+    <message>
+        <source>Copy transaction ID</source>
+        <translation>Copier l'ID de transaction</translation>
+    </message>
     <message>
         <source>Comma separated file (*.csv)</source>
         <translation>Valeurs séparées par des virgules (*.csv)</translation>
@@ -1610,6 +1670,10 @@
         <source>Date</source>
         <translation>Date</translation>
     </message>
+    <message>
+        <source>Type</source>
+        <translation>Type</translation>
+    </message>
     <message>
         <source>Label</source>
         <translation>Étiquette</translation>
@@ -1618,11 +1682,19 @@
         <source>Address</source>
         <translation>Adresse</translation>
     </message>
+    <message>
+        <source>ID</source>
+        <translation>ID</translation>
+    </message>
     <message>
         <source>Exporting Failed</source>
         <translation>Échec de l'export</translation>
     </message>
-    </context>
+    <message>
+        <source>to</source>
+        <translation>à</translation>
+    </message>
+</context>
 <context>
     <name>UnitDisplayStatusBarControl</name>
     </context>
