@@ -41,9 +41,61 @@
         <source>&amp;Delete</source>
         <translation>&amp;Şterge</translation>
     </message>
+    <message>
+        <source>C&amp;hoose</source>
+        <translation>&amp;Alegeţi</translation>
+    </message>
+    <message>
+        <source>Sending addresses</source>
+        <translation>Adresa destinatarului</translation>
+    </message>
+    <message>
+        <source>Receiving addresses</source>
+        <translation>Adresa de primire</translation>
+    </message>
+    <message>
+        <source>These are your Bitcoin addresses for sending payments. Always check the amount and the receiving address before sending coins.</source>
+        <translation>Acestea sunt adresele tale Bitcoin pentru efectuarea platilor. Intotdeauna verifica atent suma de plata si adresa beneficiarului inainte de a trimite monede.</translation>
+    </message>
+    <message>
+        <source>These are your Bitcoin addresses for receiving payments. It is recommended to use a new receiving address for each transaction.</source>
+        <translation>Acestea sunt adresele tale Bitcoin pentru receptionarea platilor. Este recomandat sa folosesti mereu o adresa noua pentru primirea platilor.</translation>
+    </message>
+    <message>
+        <source>&amp;Copy Address</source>
+        <translation>&amp;Copiază adresa</translation>
+    </message>
+    <message>
+        <source>Copy &amp;Label</source>
+        <translation>Copiaza si eticheteaza</translation>
+    </message>
+    <message>
+        <source>&amp;Edit</source>
+        <translation>&amp;Editare</translation>
+    </message>
+    <message>
+        <source>Export Address List</source>
+        <translation>Exportă listă de adrese</translation>
+    </message>
+    <message>
+        <source>Comma separated file (*.csv)</source>
+        <translation>Fisier .csv cu separator - virgula</translation>
+    </message>
+    <message>
+        <source>Exporting Failed</source>
+        <translation>Exportarea a eșuat</translation>
+    </message>
     </context>
 <context>
     <name>AddressTableModel</name>
+    <message>
+        <source>Label</source>
+        <translation>Etichetă</translation>
+    </message>
+    <message>
+        <source>Address</source>
+        <translation>Adresă</translation>
+    </message>
     </context>
 <context>
     <name>AskPassphraseDialog</name>
@@ -63,6 +115,30 @@
         <source>Repeat new passphrase</source>
         <translation>Repetaţi noua frază de acces</translation>
     </message>
+    <message>
+        <source>Encrypt wallet</source>
+        <translation>Criptare portofel</translation>
+    </message>
+    <message>
+        <source>Unlock wallet</source>
+        <translation>Deblocare portofel</translation>
+    </message>
+    <message>
+        <source>Decrypt wallet</source>
+        <translation>Decriptare portofel</translation>
+    </message>
+    <message>
+        <source>Change passphrase</source>
+        <translation>Schimbă parola</translation>
+    </message>
+    <message>
+        <source>Confirm wallet encryption</source>
+        <translation>Confirmaţi criptarea portofelului</translation>
+    </message>
+    <message>
+        <source>Wallet encrypted</source>
+        <translation>Portofel criptat</translation>
+    </message>
     </context>
 <context>
     <name>BanTableModel</name>
@@ -253,34 +329,10 @@
         <source>%n active connection(s) to Bitcoin network</source>
         <translation><numerusform>%n conexiune activă către reţeaua Bitcoin</numerusform><numerusform>%n conexiuni active către reţeaua Bitcoin</numerusform><numerusform>%n de conexiuni active către reţeaua Bitcoin</numerusform></translation>
     </message>
-    <message>
-        <source>No block source available...</source>
-        <translation>Nici o sursă de bloc disponibilă...</translation>
-    </message>
     <message numerus="yes">
         <source>Processed %n block(s) of transaction history.</source>
         <translation><numerusform>S-a procesat %n bloc din istoricul tranzacţiilor.</numerusform><numerusform>S-au procesat %n blocuri din istoricul tranzacţiilor.</numerusform><numerusform>S-au procesat %n de blocuri din istoricul tranzacţiilor.</numerusform></translation>
     </message>
-    <message numerus="yes">
-        <source>%n hour(s)</source>
-        <translation><numerusform>%n oră</numerusform><numerusform>%n ore</numerusform><numerusform>%n ore</numerusform></translation>
-    </message>
-    <message numerus="yes">
-        <source>%n day(s)</source>
-        <translation><numerusform>%n zi</numerusform><numerusform>%n zile</numerusform><numerusform>%n de zile</numerusform></translation>
-    </message>
-    <message numerus="yes">
-        <source>%n week(s)</source>
-        <translation><numerusform>%n săptămână</numerusform><numerusform>%n săptămâni</numerusform><numerusform>%n de săptămâni</numerusform></translation>
-    </message>
-    <message>
-        <source>%1 and %2</source>
-        <translation>%1 şi %2</translation>
-    </message>
-    <message numerus="yes">
-        <source>%n year(s)</source>
-        <translation><numerusform>%n an</numerusform><numerusform>%n ani</numerusform><numerusform>%n de ani</numerusform></translation>
-    </message>
     <message>
         <source>%1 behind</source>
         <translation>%1 în urmă</translation>
@@ -359,7 +411,7 @@
         <source>Wallet is &lt;b&gt;encrypted&lt;/b&gt; and currently &lt;b&gt;locked&lt;/b&gt;</source>
         <translation>Portofelul este &lt;b&gt;criptat&lt;/b&gt; iar în momentul de faţă este &lt;b&gt;blocat&lt;/b&gt;</translation>
     </message>
-</context>
+    </context>
 <context>
     <name>CoinControlDialog</name>
     <message>
@@ -378,10 +430,6 @@
         <source>Amount:</source>
         <translation>Sumă:</translation>
     </message>
-    <message>
-        <source>Priority:</source>
-        <translation>Prioritate:</translation>
-    </message>
     <message>
         <source>Fee:</source>
         <translation>Taxă:</translation>
@@ -434,10 +482,6 @@
         <source>Confirmed</source>
         <translation>Confirmat</translation>
     </message>
-    <message>
-        <source>Priority</source>
-        <translation>Prioritate</translation>
-    </message>
     </context>
 <context>
     <name>EditAddressDialog</name>
@@ -567,6 +611,21 @@
         <translation><numerusform>(din %n GB necesar)</numerusform><numerusform>(din %n GB necesari)</numerusform><numerusform>(din %n GB necesari)</numerusform></translation>
     </message>
 </context>
+<context>
+    <name>ModalOverlay</name>
+    <message>
+        <source>Form</source>
+        <translation>Form</translation>
+    </message>
+    <message>
+        <source>Last block time</source>
+        <translation>Data ultimului bloc</translation>
+    </message>
+    <message>
+        <source>Hide</source>
+        <translation>Ascunde</translation>
+    </message>
+    </context>
 <context>
     <name>OpenURIDialog</name>
     <message>
@@ -873,11 +932,7 @@
         <source>Node/Service</source>
         <translation>Nod/Serviciu</translation>
     </message>
-    <message>
-        <source>Ping Time</source>
-        <translation>Timp ping</translation>
-    </message>
-</context>
+    </context>
 <context>
     <name>QObject</name>
     <message>
@@ -916,9 +971,20 @@
         <source>%1 ms</source>
         <translation>%1 ms</translation>
     </message>
-</context>
+    <message>
+        <source>%1 and %2</source>
+        <translation>%1 şi %2</translation>
+    </message>
+    </context>
+<context>
+    <name>QObject::QObject</name>
+    </context>
 <context>
     <name>QRImageWidget</name>
+    <message>
+        <source>&amp;Save Image...</source>
+        <translation>&amp;Salvează imaginea...</translation>
+    </message>
     </context>
 <context>
     <name>RPCConsole</name>
@@ -1082,10 +1148,6 @@
         <source>Clear console</source>
         <translation>Curăţă consola</translation>
     </message>
-    <message>
-        <source>&amp;Disconnect Node</source>
-        <translation>&amp;Deconectare nod</translation>
-    </message>
     <message>
         <source>1 &amp;hour</source>
         <translation>1 &amp;oră</translation>
@@ -1244,9 +1306,25 @@
         <source>&amp;Save Image...</source>
         <translation>&amp;Salvează imaginea...</translation>
     </message>
+    <message>
+        <source>Address</source>
+        <translation>Adresă</translation>
+    </message>
+    <message>
+        <source>Amount</source>
+        <translation>Cantitate</translation>
+    </message>
+    <message>
+        <source>Label</source>
+        <translation>Etichetă</translation>
+    </message>
     </context>
 <context>
     <name>RecentRequestsTableModel</name>
+    <message>
+        <source>Label</source>
+        <translation>Etichetă</translation>
+    </message>
     </context>
 <context>
     <name>SendCoinsDialog</name>
@@ -1282,10 +1360,6 @@
         <source>Amount:</source>
         <translation>Sumă:</translation>
     </message>
-    <message>
-        <source>Priority:</source>
-        <translation>Prioritate:</translation>
-    </message>
     <message>
         <source>Fee:</source>
         <translation>Taxă:</translation>
@@ -1334,10 +1408,6 @@
         <source>Custom:</source>
         <translation>Personalizat:</translation>
     </message>
-    <message>
-        <source>Confirmation time:</source>
-        <translation>Timp confirmare:</translation>
-    </message>
     <message>
         <source>normal</source>
         <translation>normal</translation>
@@ -1551,6 +1621,10 @@
 </context>
 <context>
     <name>TransactionDesc</name>
+    <message>
+        <source>Amount</source>
+        <translation>Cantitate</translation>
+    </message>
     </context>
 <context>
     <name>TransactionDescDialog</name>
@@ -1561,9 +1635,29 @@
     </context>
 <context>
     <name>TransactionTableModel</name>
+    <message>
+        <source>Label</source>
+        <translation>Etichetă</translation>
+    </message>
     </context>
 <context>
     <name>TransactionView</name>
+    <message>
+        <source>Comma separated file (*.csv)</source>
+        <translation>Fisier .csv cu separator - virgula</translation>
+    </message>
+    <message>
+        <source>Label</source>
+        <translation>Etichetă</translation>
+    </message>
+    <message>
+        <source>Address</source>
+        <translation>Adresă</translation>
+    </message>
+    <message>
+        <source>Exporting Failed</source>
+        <translation>Exportarea a eșuat</translation>
+    </message>
     </context>
 <context>
     <name>UnitDisplayStatusBarControl</name>
@@ -1607,10 +1701,6 @@
         <source>Run in the background as a daemon and accept commands</source>
         <translation>Rulează în fundal ca un demon şi acceptă comenzi</translation>
     </message>
-    <message>
-        <source>Accept connections from outside (default: 1 if no -proxy or -connect)</source>
-        <translation>Acceptă conexiuni din afară (implicit: 1 dacă nu se foloseşte -proxy sau -connect)</translation>
-    </message>
     <message>
         <source>Bitcoin Core</source>
         <translation>Nucleul Bitcoin</translation>
@@ -1619,10 +1709,6 @@
         <source>Bind to given address and always listen on it. Use [host]:port notation for IPv6</source>
         <translation>Ataşaţi adresei date şi ascultaţi totdeauna pe ea. Folosiţi notaţia [host]:port pentru IPv6</translation>
     </message>
-    <message>
-        <source>Distributed under the MIT software license, see the accompanying file COPYING or &lt;http://www.opensource.org/licenses/mit-license.php&gt;.</source>
-        <translation>Distribuit sub licenţa de programe MIT/X11, vezi fişierul însoţitor COPYING sau &lt;http://www.opensource.org/licenses/mit-license.php&gt;.</translation>
-    </message>
     <message>
         <source>Execute command when a wallet transaction changes (%s in cmd is replaced by TxID)</source>
         <translation>Execută comanda cînd o tranzacţie a portofelului se schimbă (%s în cmd este înlocuit de TxID)</translation>
@@ -1631,18 +1717,6 @@
         <source>Set the number of script verification threads (%u to %d, 0 = auto, &lt;0 = leave that many cores free, default: %d)</source>
         <translation>Setează numărul de thread-uri de verificare a script-urilor (%u la %d, 0 = auto, &lt;0 = lasă atîtea nuclee libere, implicit: %d)</translation>
     </message>
-    <message>
-        <source>This is a pre-release test build - use at your own risk - do not use for mining or merchant applications</source>
-        <translation>Aceasta este o versiune de test preliminară - vă asumaţi riscul folosind-o - nu folosiţi pentru minerit sau aplicaţiile comercianţilor</translation>
-    </message>
-    <message>
-        <source>Warning: The network does not appear to fully agree! Some miners appear to be experiencing issues.</source>
-        <translation>Atenţie: Reţeaua nu pare să fie de acord în totalitate! Aparent nişte mineri au probleme.</translation>
-    </message>
-    <message>
-        <source>Warning: We do not appear to fully agree with our peers! You may need to upgrade, or other nodes may need to upgrade.</source>
-        <translation>Atenţie: Aparent, nu sîntem de acord cu toţi partenerii noştri! Va trebui să faceţi o actualizare, sau alte noduri necesită actualizare.</translation>
-    </message>
     <message>
         <source>&lt;category&gt; can be:</source>
         <translation>&lt;category&gt; poate fi:</translation>
@@ -1651,10 +1725,6 @@
         <source>Block creation options:</source>
         <translation>Opţiuni creare bloc:</translation>
     </message>
-    <message>
-        <source>Connect only to the specified node(s)</source>
-        <translation>Conectare doar la nod(urile) specificate</translation>
-    </message>
     <message>
         <source>Connection options:</source>
         <translation>Opţiuni conexiune:</translation>
@@ -1763,10 +1833,6 @@
         <source>Set maximum size of high-priority/low-fee transactions in bytes (default: %d)</source>
         <translation>Setează mărimea pentru tranzacţiile prioritare/taxe mici în octeţi (implicit: %d)</translation>
     </message>
-    <message>
-        <source>This product includes software developed by the OpenSSL Project for use in the OpenSSL Toolkit &lt;https://www.openssl.org/&gt; and cryptographic software written by Eric Young and UPnP software written by Thomas Bernard.</source>
-        <translation>Acest produs include programe dezvoltate de către Proiectul OpenSSL pentru a fi folosite în OpenSSL Toolkit &lt;https://www.openssl.org/&gt; şi programe criptografice scrise de către Eric Young şi programe UPnP scrise de către Thomas Bernard.</translation>
-    </message>
     <message>
         <source>(default: %u)</source>
         <translation>(implicit: %u)</translation>
@@ -1835,10 +1901,6 @@
         <source>Transaction amount too small</source>
         <translation>Suma tranzacţionată este prea mică</translation>
     </message>
-    <message>
-        <source>Transaction amounts must be positive</source>
-        <translation>Sumele tranzacţionate trebuie să fie pozitive</translation>
-    </message>
     <message>
         <source>Transaction too large for fee policy</source>
         <translation>Tranzacţie prea mare pentru politică gratis</translation>
