@@ -41,6 +41,38 @@
         <source>&amp;Delete</source>
         <translation>&amp;Izbrisati</translation>
     </message>
+    <message>
+        <source>Choose the address to send coins to</source>
+        <translation>Izaberite adresu za slanje novčića</translation>
+    </message>
+    <message>
+        <source>Choose the address to receive coins with</source>
+        <translation>Izaberite adresu za prijem novčića</translation>
+    </message>
+    <message>
+        <source>Sending addresses</source>
+        <translation>Adresa na koju se šalje</translation>
+    </message>
+    <message>
+        <source>Receiving addresses</source>
+        <translation>Adresa na koju se prima</translation>
+    </message>
+    <message>
+        <source>These are your Bitcoin addresses for sending payments. Always check the amount and the receiving address before sending coins.</source>
+        <translation>Ovo su Vaše Bitcoin adrese na koju se vrše uplate. Uvek proverite iznos i prijemnu adresu pre slanja novčića</translation>
+    </message>
+    <message>
+        <source>These are your Bitcoin addresses for receiving payments. It is recommended to use a new receiving address for each transaction.</source>
+        <translation>Ovo su Vaše Bitcoin adrese za primanje uplata. Preporučuje se upotreba nove adrese za svaku transakciju.</translation>
+    </message>
+    <message>
+        <source>Copy &amp;Label</source>
+        <translation>Kopiranje &amp;Oznaka</translation>
+    </message>
+    <message>
+        <source>&amp;Edit</source>
+        <translation>&amp;Izmena</translation>
+    </message>
     </context>
 <context>
     <name>AddressTableModel</name>
@@ -137,6 +169,14 @@
         <source>Quit application</source>
         <translation>Isključi aplikaciju</translation>
     </message>
+    <message>
+        <source>&amp;About %1</source>
+        <translation>&amp;Otprilike %1</translation>
+    </message>
+    <message>
+        <source>Show information about %1</source>
+        <translation>Prikaži informacije za otprilike %1</translation>
+    </message>
     <message>
         <source>&amp;Options...</source>
         <translation>&amp;Opcije...</translation>
